%% Copyright (c) 2011, INRA
%% 2007-2011, David Legland <david.legland@grignon.inra.fr>
%% 2011 Adapted to Octave by Juan Pablo Carbajal <carbajal@ifi.uzh.ch>
%%
%% All rights reserved.
%% (simplified BSD License)
%%
%% Redistribution and use in source and binary forms, with or without
%% modification, are permitted provided that the following conditions are met:
%%
%% 1. Redistributions of source code must retain the above copyright notice, this
%%    list of conditions and the following disclaimer.
%%     
%% 2. Redistributions in binary form must reproduce the above copyright notice, 
%%    this list of conditions and the following disclaimer in the documentation
%%    and/or other materials provided with the distribution.
%%
%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
%% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
%% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
%% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
%% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
%% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
%% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
%% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
%% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
%% POSSIBILITY OF SUCH DAMAGE.
%%
%% The views and conclusions contained in the software and documentation are
%% those of the authors and should not be interpreted as representing official
%% policies, either expressed or implied, of copyright holder.

%% -*- texinfo -*-
%% @deftypefn {Function File} {@var{dist} = } distancePointEdge (@var{point}, @var{edge})
%% Minimum distance between a point and an edge
%%
%%   DIST = distancePointEdge(POINT, EDGE);
%%   Return the euclidean distance between edge EDGE and point POINT. 
%%   EDGE has the form: [x1 y1 x2 y2], and POINT is [x y].
%%
%%   If EDGE is N-by-4 array, result is N-by-1 array computed for each edge.
%%   If POINT is a N-by-2 array, the result is computed for each point.
%%   If both POINT and EDGE are array, they must have the same number of
%%   rows, and the result is computed for each couple point(i,:);edge(i,:).
%%
%%   [DIST POS] = distancePointEdge(POINT, EDGE);
%%   Also returns the position of closest point on the edge. POS is
%%   comprised between 0 (first point) and 1 (last point).
%%
%%   @seealso{edges2d, points2d, distancePoints, distancePointLine}
%% @end deftypefn

function varargout = distancePointEdge(point, edge)
  % direction vector of each edge
  dx = edge(:, 3) - edge(:,1);
  dy = edge(:, 4) - edge(:,2);

  % compute position of points projected on the supporting line
  % (Size of tp is the max number of edges or points)
  delta = dx .* dx + dy .* dy;
  tp = ((point(:, 1) - edge(:, 1)) .* dx + (point(:, 2) - edge(:, 2)) .* dy) ./ delta;

  % ensure degenerated edges are correclty processed (consider the first
  % vertex is the closest)
  tp(delta < eps) = 0;

  % change position to ensure projected point is located on the edge
  tp(tp < 0) = 0;
  tp(tp > 1) = 1;

  % coordinates of projected point
  p0 = [edge(:,1) + tp .* dx, edge(:,2) + tp .* dy];

  % compute distance between point and its projection on the edge
  dist = sqrt((point(:,1) - p0(:,1)) .^ 2 + (point(:,2) - p0(:,2)) .^ 2);

  % process output arguments
  varargout{1} = dist;
  if nargout > 1
      varargout{2} = tp;
  end

endfunction

